@@ -149,6 +149,8 @@ wireToPropertyEvent(xXIPropertyEvent *in, XGenericEventCookie *cookie);
 
 static /* const */ XEvent emptyevent;
 
+typedef Status (*core_event_to_wire)(Display*, XEvent*, xEvent*);
+
 static /* const */ XExtensionHooks xinput_extension_hooks = {
     NULL,	/* create_gc */
     NULL,	/* copy_gc */
@@ -158,7 +160,7 @@ static /* const */ XExtensionHooks xinput_extension_hooks = {
     NULL,	/* free_font */
     XInputClose,	/* close_display */
     XInputWireToEvent,	/* wire_to_event */
-    _XiEventToWire,	/* event_to_wire */
+    (core_event_to_wire)_XiEventToWire, /* event_to_wire */
     NULL,	/* error */
     XInputError,	/* error_string */
 };
@@ -268,7 +270,8 @@ static XExtensionVersion versions[] = { {XI_Absent, 0, 0},
  XI_Add_DevicePresenceNotify_Minor},
 {XI_Present, XI_Add_DeviceProperties_Major,
  XI_Add_DeviceProperties_Minor},
-{XI_Present, 2, 0}
+{XI_Present, 2, 0},
+{XI_Present, 2, 1}
 };
 
 /***********************************************************************
@@ -760,7 +763,6 @@ XInputWireToEvent(
                         return (DONT_ENQUEUE);
                     else {
                         *re = *save;
-                        stev = (XDeviceStateNotifyEvent *) re;
                         return (ENQUEUE_EVENT);
                     }
                 }
@@ -788,7 +790,6 @@ XInputWireToEvent(
                         return (DONT_ENQUEUE);
                     else {
                         *re = *save;
-                        kstev = (XDeviceStateNotifyEvent *) re;
                         return (ENQUEUE_EVENT);
                     }
                 }
@@ -816,7 +817,6 @@ XInputWireToEvent(
                         return (DONT_ENQUEUE);
                     else {
                         *re = *save;
-                        bstev = (XDeviceStateNotifyEvent *) re;
                         return (ENQUEUE_EVENT);
                     }
                 }
@@ -1039,6 +1039,9 @@ sizeDeviceClassType(int type, int num_elements)
         case XIValuatorClass:
             l = sizeof(XIValuatorClassInfo);
             break;
+        case XIScrollClass:
+            l = sizeof(XIScrollClassInfo);
+            break;
         default:
             printf("sizeDeviceClassType: unknown type %d\n", type);
             break;
@@ -1098,6 +1101,9 @@ copyDeviceChangedEvent(XGenericEventCookie *in_cookie,
             case XIValuatorClass:
                 len += sizeDeviceClassType(XIValuatorClass, 0);
                 break;
+            case XIScrollClass:
+                len += sizeDeviceClassType(XIScrollClass, 0);
+                break;
             default:
                 printf("copyDeviceChangedEvent: unknown type %d\n",
                         any->type);
@@ -1162,6 +1168,15 @@ copyDeviceChangedEvent(XGenericEventCookie *in_cookie,
                     out->classes[i] = (XIAnyClassInfo*)vout;
                     break;
                 }
+            case XIScrollClass:
+                {
+                    XIScrollClassInfo *sin, *sout;
+                    sin = (XIScrollClassInfo*)any;
+                    sout = next_block(&ptr, sizeof(XIScrollClassInfo));
+                    *sout = *sin;
+                    out->classes[i] = (XIAnyClassInfo*)sout;
+                    break;
+                }
         }
     }
 
@@ -1433,6 +1448,9 @@ size_classes(xXIAnyInfo* from, int nclasses)
             case XIValuatorClass:
                 l = sizeDeviceClassType(XIValuatorClass, 0);
                 break;
+            case XIScrollClass:
+                l = sizeDeviceClassType(XIScrollClass, 0);
+                break;
         }
 
         len += l;
@@ -1463,8 +1481,8 @@ copy_classes(XIDeviceInfo* to, xXIAnyInfo* from, int *nclasses)
 
     ptr_wire = (char*)from;
     ptr_lib = to->classes;
-    to->classes = next_block(&ptr_lib, (*nclasses) * sizeof(XIAnyClassInfo*));
-    memset(to->classes, 0, (*nclasses) * sizeof(XIAnyClassInfo*));
+    to->classes = next_block(&ptr_lib, *nclasses * sizeof(XIAnyClassInfo*));
+    memset(to->classes, 0, sizeof(*nclasses * sizeof(XIAnyClassInfo*)));
     len = 0; /* count wire length */
 
     for (i = 0; i < *nclasses; i++)
@@ -1545,6 +1563,25 @@ copy_classes(XIDeviceInfo* to, xXIAnyInfo* from, int *nclasses)
                     to->classes[cls_idx++] = any_lib;
                 }
                 break;
+            case XIScrollClass:
+                {
+                    XIScrollClassInfo *cls_lib;
+                    xXIScrollInfo *cls_wire;
+
+                    cls_lib = next_block(&ptr_lib, sizeof(XIScrollClassInfo));
+                    cls_wire = (xXIScrollInfo*)any_wire;
+
+                    cls_lib->type = cls_wire->type;
+                    cls_lib->sourceid = cls_wire->sourceid;
+                    cls_lib->number     = cls_wire->number;
+                    cls_lib->scroll_type= cls_wire->scroll_type;
+                    cls_lib->flags      = cls_wire->flags;
+                    cls_lib->increment  = cls_wire->increment.integral;
+                    cls_lib->increment += (unsigned int)cls_wire->increment.frac/(double)(1UL << 32);
+
+                    to->classes[cls_idx++] = any_lib;
+                }
+                break;
         }
         len += any_wire->length * 4;
         ptr_wire += any_wire->length * 4;
